
/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/


#include "netinc.h"

#include "neterr.h"
#include "commands.h"
#include "iowrap.h"
#include "netio.h"
#include "interval.h"
#include "netfile.h"


/*
 * get_io_block
 *

   Read an io_block structure (see netio.h) from the socket and return
   the command that starts the block.

   All data blocks have a command field and a length embedded in
   them.  

   For higher performance there is no acknowledge between the sender
   and the receiver.  This makes things a bit more complicated, since
   in rare instances partial data (e.g., just the command word) may
   be sent.

 */

COMMANDS netfile::get_io_block( int sockfd, io_block &blk, 
				const char *prog_name)
{
  int amt_read;
  int data_read;
  
  if (debug)
    printf("get_io_block\n");
  
  amt_read = read_block( sockfd, (char *)&blk, sizeof(blk) );

  if (endian_mismatch()) {
    blk.com = (COMMANDS)byte_flip( (unsigned int )blk.com );
  }

  // take care of the rare case when just the command word is sent
  // and the header is incomplete.
  if (blk.com == sending_data && amt_read < IO_BUF_EXTRA) {
    int amt_read_2nd_time;
    int size;
    char *pBlk;
    
    pBlk = (char *)&blk;
    pBlk = pBlk + amt_read;
    size = sizeof(blk) - amt_read;
    
    amt_read_2nd_time = read_block( sockfd, pBlk, size );

    if (debug) {
      printf("get_io_block: short data block, amt_read = %d\n", amt_read );
      printf("get_io_block: amt_read_2nd_time = %d, total = %d\n",
	     amt_read, amt_read + amt_read_2nd_time );
    }

    amt_read = amt_read + amt_read_2nd_time;
  }
  
  if (endian_mismatch() && blk.size != 0) {
    blk.size = byte_flip( blk.size );
  }
  
  if (debug) {
    client_commands com_obj;
    const char *com_str;
    
    com_str = com_obj.command_name( blk.com );
    printf("get_io_block: command (0x%x) = %s, amt_read = %d, blk.size = %d\n",
	   blk.com, com_str, amt_read, blk.size );
    
  }

  if (blk.com == sending_data) {
    // the io_block structure contains command and size information.
    // data_read is the amount of data read with this factored out.

    if (amt_read > IO_BUF_EXTRA)
      data_read = amt_read - IO_BUF_EXTRA;
    else
      data_read = 0;
    
    if (data_read < blk.size) {
      // we don't have all the data, so read more from the socket
      char *pChar;
      int amt_to_read;
      int bytes_read;
      int total_bytes;

      total_bytes = amt_read;
      
      bytes_read = 0;
      amt_to_read = blk.size - data_read;

      if (debug) {
	printf("get_io_block: data_read (%d) < blk.size (%d)\n",
	       data_read, blk.size );
      }
      
      for (pChar = (char *)&blk + amt_read;
	   amt_to_read > 0;
	   amt_to_read = amt_to_read - bytes_read) {
	
	bytes_read = read_block( sockfd, pChar, amt_to_read );
	total_bytes = total_bytes + bytes_read;

	if (debug) {
	  printf("get_io_block: amt_to_read = %d, bytes_read = %d, total_read = %d\n",
		 amt_to_read, bytes_read, total_bytes );
	}
	
	pChar = pChar + bytes_read;
	if (bytes_read > amt_to_read) {
	  printf("%s: get_io_block - amt_read (%d) > amt_to_read (%d)\n",
		 prog_name, amt_read, amt_to_read);
	}
	
      } // for
    }
  } // if blk.com == sending_data
    
  return blk.com;
} // get_io_block



/*
 * SelectWrite
 *

   Wait on sockfd until the timeout is reached or until
   the socket is ready for another write operation.

   The function returns a SELECT_RSLT.

 */
SELECT_RSLT netfile::SelectWrite( const int sockfd, const char *prog_name )
{
  fd_set write_fd, except_fd;
  struct timeval timeout;
  int select_rslt;
  SELECT_RSLT rslt;

  FD_ZERO( &write_fd );
  FD_ZERO( &except_fd );
  timeout.tv_usec = 0;
  timeout.tv_sec = 5; // set for 56K modem delay

  FD_SET( sockfd, &write_fd );
  FD_SET( sockfd, &except_fd );

  select_rslt = select( sockfd + 1, // for UNIX
			NULL,       // no readfds
			&write_fd,
			&except_fd,
			&timeout );

  rslt = select_bad_rslt;
  if (select_rslt == 1) { // one file descriptor is ready to go
    if (FD_ISSET( sockfd, &write_fd)) {
      rslt = select_ok_to_write;
    }
    else {
      printf("%s: SelectWrite - fd is not set\n", prog_name );
    }

    if (FD_ISSET( sockfd, &except_fd)) {
      printf("%s: except_fd is set\n", prog_name );
    }
  }
  else if (select_rslt == 0) { // select timed out
    rslt = select_write_timeout;
    printf("%s: socket time out\n", prog_name );
  }
  else if (select_rslt == SOCKET_ERROR) {
    char buf[128];

    sprintf(buf, "%s: SelectWrite socket error", prog_name );
    PrintErr( buf );
    rslt = select_write_error;
  }

  return rslt;
} // SelectWrite



/*
 * hex_dump_word
 *
 */
void netfile::hex_dump_word( unsigned int word )
{
  unsigned char byte1, byte2, byte3, byte4;

  byte1 = word;
  byte2 = word >> 8;
  byte3 = word >> 16;
  byte4 = word >> 24;
  printf("%02x%02x%02x%02x", byte1, byte2, byte3, byte4 );
} // hex_dump_word



/*
 * hex_dump
 *
 */
void netfile::hex_dump( io_block &blk )
{
  const int words_per_line = 16;
  int i, num_words;
  unsigned int *pWord;
  Boolean CR;

  num_words = blk.size / sizeof( int );
  pWord = (unsigned int *)blk.buf;
  for (i = 0; i < num_words; i++) {
    CR = FALSE;
    hex_dump_word( pWord[i] );
    if ((i+1) % words_per_line == 0) {
      printf("\n");
      CR = TRUE;
    }
    else
      printf(" ");
  }
  if (! CR)
    printf("\n");
} // hex_dump



/*
 * get_file
 *

   Get a file from the socket sockfd and write it to the file
   associated with dest_fp.

   The function returns the number of bytes written to the file

 */
int netfile::get_file( int sockfd, FILE *dest_fp, 
			const char *file_name, const char *prog_name )
{
  io_block blk;
  iowrap fileio;
  COMMANDS com;
  int num_bytes;

  if (debug)
    printf("get_file\n");

  num_bytes = 0;
  while ((com = get_io_block( sockfd, blk, prog_name )) == sending_data) {

    if (! fileio.Fwrite( blk.buf, blk.size, dest_fp, file_name, prog_name )) {
      break;
    }
    else {
      num_bytes = num_bytes + blk.size;
    }

    blk.com = bad_command;
    blk.size = 0;
  } // while

  return num_bytes;
} // get_file



/*
 * put_file
 *

   Send a file from the client to the server.

   This function attempts to time the transfer and report
   the data rate.  If the file being sent is small, the send
   is non-blocking and returns quickly.  Since time is only 
   measured in milliseconds (which is a long time on a modern
   computer), for small files the send time may be "zero".  In
   this case the amount of data is reported, not the data
   rate.

   If the last block sent is smaller than blk.buf, the send_done
   command may be appended.  This function rounds the amount of data
   sent to the nearest word so that the command will be aligned
   on a word boundary when its fetched.

 */
void netfile::put_file( int sockfd, FILE *src_fp, const char *srcname,
			const char *prog_name )
{
  io_block blk;
  unsigned int amt_transfered;
  int amt_read, amt_to_send;
  time_interval timer;
  elapsed_time delta_t;
  COMMANDS com_in;
  iowrap fileio;

  if (debug)
    printf("put_file\n");
  
  amt_transfered = 0;
  timer.start_time();
  while ((amt_read = fileio.Fread( blk.buf, sizeof( blk.buf ), src_fp, 
				   srcname, prog_name )) > 0) {

    if (debug)
      printf("put_file: send_io_block\n");

    send_io_block( sockfd, blk, amt_read );

    amt_transfered = amt_transfered + amt_read;
  }

  if (debug)
    printf("put_file: SendCommand( send_done )\n");
  SendCommand( sockfd, send_done );  

  // report transfer speed
  if (amt_transfered > 0) {
    int transfer_rate;

    timer.end_time();
    delta_t = timer.get_elapsed();
    transfer_rate = 0;

    if (delta_t.sec > 0) {
      transfer_rate = amt_transfered / delta_t.sec;
    }
    else {
      /* conversion for byte/millisec to bytes/sec

           bytes       1000 millisec
           -----    X  -------------
           millisec       1 sec
       */

      if (delta_t.millisec > 0)  // avoid div by zero
	transfer_rate = (amt_transfered * 1000)/delta_t.millisec;
    }
    printf("%d bytes transfered", amt_transfered );
    if (delta_t.sec > 0) {
      printf(" in %d seconds", delta_t.sec );
    }
    else if (delta_t.millisec > 0) {
      printf(" in %d milliseconds", delta_t.millisec );
    }
    if (transfer_rate > 0) {
      printf(" (%d bytes/sec)", transfer_rate);
    }
    printf("\n");
  }
} // put_file

