
/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/

#include "netinc.h"

#include "neterr.h"
#include "netwrap.h"
#include "iowrap.h"
#include "commands.h"
#include "netio.h"
#include "interval.h"
#include "netfile.h"
#include "netserv.h"
#include "switch.h"


/*
 * WinInit
 *

   WinSock initialization.

 */
void switchboard::WinInit(void)
{
#if WIN32  
  WSADATA wsaData; 

  if (WSAStartup(0x202,&wsaData) == SOCKET_ERROR) { 
    fprintf(stderr,"WSAStartup failed with error %d\n",
	    WSAGetLastError()); 
    WSACleanup(); 
    exit( -1 ); 
  } 
#endif
}  // WinInit



/*
 * get_hostent
 *
 */
struct hostent *switchboard::get_hostent( const char *server_name )
{
  struct hostent *hp;
  neterr err;

  // 
  // Attempt to detect if we should call gethostbyname() or 
  // gethostbyaddr() 
  
  if (isalpha(server_name[0])) { /* server address is a name */ 
    hp = gethostbyname(server_name); 
  } 
  else  {		/* Convert nnn.nnn address to a usable one */ 
    unsigned int addr; 

    addr = inet_addr(server_name); 
    hp = gethostbyaddr((char *)&addr,4,AF_INET); 
  } 
  if (hp == NULL ) { 
    char buf[128];

    sprintf(buf, "Client: Cannot resolve address [%s]:", server_name); 
    err.Error( buf );
  } 

  return hp;
} // get_hostent



/*
 * print_inet_addr
 *

   Print an IP address in "dot format" (e.g., 192.125.34.27).

   The function is passed a pointer to an array of bytes, where
   each byte contains one part of the IP address.

 */
void switchboard::print_inet_addr( const char *pCh, const int len )
{
  int i;

  for (i = 0; i < len; i++, pCh++) {
    printf("%d", *(unsigned char *)pCh );
    if (i < 3)
      printf(".");
  }
  printf("\n");
} // print_inet_addr



void switchboard::print_hostent( struct hostent *pHostent )
{
  char **pAddrList;

  printf("pHostent->h_length = %d\n", pHostent->h_length );
  printf("pHostent->h_addr = 0x%x\n", pHostent->h_addr );
  print_inet_addr( pHostent->h_addr, pHostent->h_length );

  pAddrList = pHostent->h_addr_list;
  while (*pAddrList != NULL) {
    print_inet_addr( *pAddrList, pHostent->h_length );
    pAddrList++;
  }
} // print_hostent



/*
 * connect_message
 *

   Print a connect message

 */
void switchboard::connect_message(void)
{
  if (pHostent != NULL) {
    printf("Connected to %s at ", pHostent->h_name );
    print_inet_addr( pHostent->h_addr, pHostent->h_length );
  }
  else {
    printf("connect_message: pHostent is NULL\n");
  }
} // connect_message



/*
 * set_sockaddr
 *
   Initialize the sockaddr_in data structure
 */
void switchboard::set_sockaddr( struct sockaddr_in *pSockAddr, 
				int family, unsigned int port )
{
  memset( pSockAddr, 0, sizeof(struct sockaddr_in) );
  pSockAddr->sin_family = AF_INET;
  pSockAddr->sin_addr.s_addr = htonl(INADDR_ANY);
  pSockAddr->sin_port = htons(port);
} // set_sockaddr



/*
 * server_accept
 *

   Listen on "port".  Errors will terminate the program.
   
   The function returns the socket file descriptor (fd)
   that results from accepting a connection from the client.

 */
int switchboard::server_accept( const unsigned int port )
{
  int listenfd, connfd;
  int clilen;
  struct sockaddr_in servaddr, cliaddr;
  netwrap netfunc;

  WinInit();

  listenfd = Socket( AF_INET, SOCK_STREAM, 0);

  set_sockaddr( &servaddr, AF_INET, port );
  
  Bind( listenfd, (struct sockaddr*)&servaddr, sizeof( servaddr) );

  Listen( listenfd, BACKLOG );

  clilen = sizeof( cliaddr );

  connfd = netfunc.Accept( listenfd, (struct sockaddr*)&cliaddr, &clilen);

  return connfd;
}  // server_accept



/*
 * client_connect
 *

   Connect to the server on the system "sys_name", using "port".

   The function returns the socket file descriptor (fd).

 */
int switchboard::client_connect( const unsigned int port,
				 const char *sys_name )
{
  int sockfd;
  struct sockaddr_in servaddr;

  WinInit();

  memset( &servaddr, 0, sizeof( servaddr ));
  
  sockfd = Socket( AF_INET, SOCK_STREAM, 0);
  set_sockaddr( &servaddr, AF_INET, port );  

  pHostent = get_hostent( sys_name );

  // copy the four byte IP address from the hostent data structure
  // to the sin_addr field.
  memcpy(&servaddr.sin_addr, pHostent->h_addr, pHostent->h_length );
  
  Connect( sockfd, (struct sockaddr *)&servaddr, sizeof( servaddr ) );

  return sockfd;
} // client_connect
